// not part of package InfoCollection
import InfoCollection.SamplingRule;
import InfoCollection.StoppingRuleSeq;
import InfoCollection.Simulator;
import InfoCollection.TestExamples;
import InfoCollection.NormalTruth;
import InfoCollection.Truth;
import InfoCollection.FixedStop;
import InfoCollection.IndependentNormalVarianceUnknown.Belief;
import InfoCollection.IndependentNormalVarianceUnknown.LL1;
import InfoCollection.IndependentNormalVarianceUnknown.ChIn;
import InfoCollection.IndependentNormalVarianceUnknown.KGStoppingRule;
import InfoCollection.IndependentNormalVarianceUnknown.EOCBonfStoppingRule;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;

public class Sim20080214 {
	/* This is the basic code for running the experiment. */
	private static void Run(SamplingRule policy, StoppingRuleSeq stopSeq, Truth truth) {
		int nruns = 100000; // for production
		//int nruns = 1000; // for testing
		int maxN = 2000;
		double result[][];

		/* The description of truth will have newlines in it, so we
		 * have to replace these with comments. */
		String truthDescrip = "# " + truth.toString();
		truthDescrip = truthDescrip.replaceAll("\n", "\n# ");

		System.out.printf("# nruns=%d maxN=%d\n", nruns, maxN);
		System.out.println("# Policy=" + policy);
		System.out.println(truthDescrip);
		System.out.println("# Stopping Rule=" + stopSeq);
		System.out.println("# Stopping Rule=" + stopSeq);
		System.out.println("# E[N]\tE[OC]\ttimeouts");

		try {
			result = Simulator.LossWhenStopped(policy, truth, stopSeq, nruns, maxN);
		} catch (Exception e) {
			System.err.println("Internal Error");
			e.printStackTrace();
			return;
		}
			
		for (int s=0; s<stopSeq.Length(); s++)
			System.out.printf("%.6f\t%.6f\t%d\n",
					result[s][1], result[s][0], (int)result[s][2]);
	}

	/*
	 * With this code, we can run the experiment while also collecting
	 * error bars.
	 */
/*
	private static void RunWithErrors(SamplingRule policy, StoppingRuleSeq stopSeq, Truth truth) {
		int runsPerBlock = 100;
		int i, nblocks = 100;
		int maxN = 2000;
		double result[][];
		MeanVariance EN[], EOC[];

		System.out.printf("# nBlocks=%d runsPerBlock=%d maxN=%d\n",
				nBlocks, runsPerBlock, maxN);
		System.out.println("# Policy=" + policy);
		System.out.println("# Stopping Rule=" + stopSeq);
		System.out.println("# Stopping Rule=" + stopSeq);
		System.out.println("# E[N](sample mean, stderr)\tE[OC](sample mean, stderr)\ttimeouts");
		EN = new MeanVariance[stopSeq.length];
		EOC = new MeanVariance[stopSeq.length];
		for (i=0; i<stopSeq.length; i++) {
			EN[i] = new MeanVariance();
			EOC[i] = new MeanVariance();
		}
		try {
			for (i=0; i<nblocks; i++) {
				// Run the block, then unpack the result.
				result = Simulator.LossWhenStopped(policy, truth, stopSeq, runsPerBlock, maxN);
EN[i].addSample();
EOC[i].addSample();
			}
		} catch (Exception e) {
			System.err.println("Internal Error");
			e.printStackTrace();
			return;
		}
			
		for (int s=0; s<stopSeq.Length(); s++)
			System.out.printf("%.3f\t%.3f\t%d\n",
					result[s][1], result[s][0], (int)result[s][2]);
	}
*/

	public static void main(String args[]) {
		if (args.length == 1) {
			runtest(args[0]);
		} else
			System.err.println("usage: Sim20080214 test_num");
	}

	/*
	 * Run test number n. This is a function named "testn", where n
	 * is filled in with the test number.  Returns 0 if the test succeeded,
	 * -1 if the test was not found, and -2 if the test was found but
	 * failed.  Also prints an error message if it failed.  It may also
	 * fail for some unexpected reason, in which case it throws that
	 * exception.
	 */
	public static void runtest(String suffix) {
		Method method = null;
		String testname = "test" + suffix;
		try {
			method = Sim20080214.class.getDeclaredMethod(testname, new Class[] {});
		} catch(NoSuchMethodException e) {
			System.err.printf("%s not found\n", testname);
		}
		try {
			method.invoke(null);
		} catch(InvocationTargetException target_e) {
			target_e.getCause().printStackTrace();
			System.err.println(testname + " failed");
		} catch(IllegalAccessException e) {
			e.printStackTrace();
			System.err.println(testname + " access error");
		}
	}


	/*
	 * This first set of tests are of the slippage configuration.  The first
	 * three are LL1 with different stopping rules.  The next three are LL
	 * with different stopping rules.
	 */
	public static void test1() {
		//SamplingRule sampler = new LL1();
		SamplingRule sampler = new LL1();
		StoppingRuleSeq stopSeq = KGStoppingRule.Seq();
		Truth truth = NormalTruth.SlippageExample();
		Run(sampler, stopSeq, truth);
	}
	public static void test2() {
		SamplingRule sampler = new LL1();
		StoppingRuleSeq stopSeq = FixedStop.Seq();
		Truth truth = NormalTruth.SlippageExample();
		Run(sampler, stopSeq, truth);
	}
	public static void test3() {
		SamplingRule sampler = new LL1();
		StoppingRuleSeq stopSeq = EOCBonfStoppingRule.Seq();
		Truth truth = NormalTruth.SlippageExample();
		Run(sampler, stopSeq, truth);
	}
	public static void test4() {
		SamplingRule sampler = new ChIn();
		StoppingRuleSeq stopSeq = KGStoppingRule.Seq();
		Truth truth = NormalTruth.SlippageExample();
		Run(sampler, stopSeq, truth);
	}
	public static void test5() {
		SamplingRule sampler = new ChIn();
		StoppingRuleSeq stopSeq = FixedStop.Seq();
		Truth truth = NormalTruth.SlippageExample();
		Run(sampler, stopSeq, truth);
	}
	public static void test6() {
		SamplingRule sampler = new ChIn();
		StoppingRuleSeq stopSeq = EOCBonfStoppingRule.Seq();
		Truth truth = NormalTruth.SlippageExample();
		Run(sampler, stopSeq, truth);
	}


	/*
	 * This next set of tests is the monotone-decreasing-means (MDM)
	 * configuration.  As with the SC tests 1-6 above, the first three are
	 * LL1 with different stopping rules.  The next three are LL with
	 * different stopping rules.
	 */
	public static void test7() {
		SamplingRule sampler = new LL1();
		StoppingRuleSeq stopSeq = KGStoppingRule.Seq();
		Truth truth = NormalTruth.MonotoneDecreasingMeansExample();
		Run(sampler, stopSeq, truth);
	}
	public static void test8() {
		SamplingRule sampler = new LL1();
		StoppingRuleSeq stopSeq = FixedStop.Seq();
		Truth truth = NormalTruth.MonotoneDecreasingMeansExample();
		Run(sampler, stopSeq, truth);
	}
	public static void test9() {
		SamplingRule sampler = new LL1();
		StoppingRuleSeq stopSeq = EOCBonfStoppingRule.Seq();
		Truth truth = NormalTruth.MonotoneDecreasingMeansExample();
		Run(sampler, stopSeq, truth);
	}
	public static void test10() {
		SamplingRule sampler = new ChIn();
		StoppingRuleSeq stopSeq = KGStoppingRule.Seq();
		Truth truth = NormalTruth.MonotoneDecreasingMeansExample();
		Run(sampler, stopSeq, truth);
	}
	public static void test11() {
		SamplingRule sampler = new ChIn();
		StoppingRuleSeq stopSeq = FixedStop.Seq();
		Truth truth = NormalTruth.MonotoneDecreasingMeansExample();
		Run(sampler, stopSeq, truth);
	}
	public static void test12() {
		SamplingRule sampler = new ChIn();
		StoppingRuleSeq stopSeq = EOCBonfStoppingRule.Seq();
		Truth truth = NormalTruth.MonotoneDecreasingMeansExample();
		Run(sampler, stopSeq, truth);
	}

	/*
	 * And now we have the exponentially decreasing means configuration.
	 * As with the SC and MDM tests above, the first three are LL1 with
	 * different stopping rules.  The next three are LL with different
	 * stopping rules.  Note that we have not (yet) reproduced these 6
	 * tests in A* and B* with the variance known policies.
	 *
	 * For this EDM configuration, the default levels of logc used for KG
	 * stopping has some values that are too large negative.  Thus we
	 * provide our own set of logc levels.  Also, the EOCBonf default
	 * values are also not right, so we provide our own levels here as
	 * well.
	 */
	public static StoppingRuleSeq EDMKGSeq() {
		double logc[] = {-1,-2,-3,-4,-5,-6,-7,-8,-9,-10,-11,-12,-13,-14,-15,-16,-17,-18,-19,-20,-21,-22,-23,-24,-25,-26,-27,-28,-29,-30};
		return KGStoppingRule.Seq(logc);
	}
	public static StoppingRuleSeq EDMEOCBonfSeq() {
		double threshold[] = {100,80,75,60,50,40,35,30,25,20,15,10,7,6,5,4,3,2.5,2,1.5,1};
		return EOCBonfStoppingRule.Seq(threshold);
	}

	public static void test13() {
		SamplingRule sampler = new LL1();
		StoppingRuleSeq stopSeq = EDMKGSeq();
		Truth truth = NormalTruth.ExponentialDecreasingMeansExample();
		Run(sampler, stopSeq, truth);
	}
	public static void test14() {
		SamplingRule sampler = new LL1();
		StoppingRuleSeq stopSeq = FixedStop.Seq();
		Truth truth = NormalTruth.ExponentialDecreasingMeansExample();
		Run(sampler, stopSeq, truth);
	}
	public static void test15() {
		SamplingRule sampler = new LL1();
		StoppingRuleSeq stopSeq = EDMEOCBonfSeq();
		Truth truth = NormalTruth.ExponentialDecreasingMeansExample();
		Run(sampler, stopSeq, truth);
	}
	public static void test16() {
		SamplingRule sampler = new ChIn();
		StoppingRuleSeq stopSeq = EDMKGSeq();
		Truth truth = NormalTruth.ExponentialDecreasingMeansExample();
		Run(sampler, stopSeq, truth);
	}
	public static void test17() {
		SamplingRule sampler = new ChIn();
		StoppingRuleSeq stopSeq = FixedStop.Seq();
		Truth truth = NormalTruth.ExponentialDecreasingMeansExample();
		Run(sampler, stopSeq, truth);
	}
	public static void test18() {
		SamplingRule sampler = new ChIn();
		StoppingRuleSeq stopSeq = EDMEOCBonfSeq();
		Truth truth = NormalTruth.ExponentialDecreasingMeansExample();
		Run(sampler, stopSeq, truth);
	}



	/*
	 * These are exactly the same as test1 through test12, but with the
	 * variance-known versions of the sampling and stopping rules.
	 */
	public static void testA1() {
		SamplingRule sampler = new InfoCollection.IndependentNormalVarianceKnown.KnowledgeGradient();
		StoppingRuleSeq stopSeq = InfoCollection.IndependentNormalVarianceKnown.KGStoppingRule.Seq();
		Truth truth = NormalTruth.SlippageExample();
		Run(sampler, stopSeq, truth);
	}
	public static void testA2() {
		SamplingRule sampler = new InfoCollection.IndependentNormalVarianceKnown.KnowledgeGradient();
		StoppingRuleSeq stopSeq = FixedStop.Seq();
		Truth truth = NormalTruth.SlippageExample();
		Run(sampler, stopSeq, truth);
	}
	public static void testA3() {
		SamplingRule sampler = new InfoCollection.IndependentNormalVarianceKnown.KnowledgeGradient();
		StoppingRuleSeq stopSeq = InfoCollection.IndependentNormalVarianceKnown.EOCBonfStoppingRule.Seq();
		Truth truth = NormalTruth.SlippageExample();
		Run(sampler, stopSeq, truth);
	}
	public static void testA4() {
		SamplingRule sampler = new InfoCollection.IndependentNormalVarianceKnown.ChIn();
		StoppingRuleSeq stopSeq = InfoCollection.IndependentNormalVarianceKnown.KGStoppingRule.Seq();
		Truth truth = NormalTruth.SlippageExample();
		Run(sampler, stopSeq, truth);
	}
	public static void testA5() {
		SamplingRule sampler = new InfoCollection.IndependentNormalVarianceKnown.ChIn();
		StoppingRuleSeq stopSeq = FixedStop.Seq();
		Truth truth = NormalTruth.SlippageExample();
		Run(sampler, stopSeq, truth);
	}
	public static void testA6() {
		SamplingRule sampler = new InfoCollection.IndependentNormalVarianceKnown.ChIn();
		StoppingRuleSeq stopSeq = InfoCollection.IndependentNormalVarianceKnown.EOCBonfStoppingRule.Seq();
		Truth truth = NormalTruth.SlippageExample();
		Run(sampler, stopSeq, truth);
	}
	public static void testA7() {
		SamplingRule sampler = new InfoCollection.IndependentNormalVarianceKnown.KnowledgeGradient();
		StoppingRuleSeq stopSeq = InfoCollection.IndependentNormalVarianceKnown.KGStoppingRule.Seq();
		Truth truth = NormalTruth.MonotoneDecreasingMeansExample();
		Run(sampler, stopSeq, truth);
	}
	public static void testA8() {
		SamplingRule sampler = new InfoCollection.IndependentNormalVarianceKnown.KnowledgeGradient();
		StoppingRuleSeq stopSeq = FixedStop.Seq();
		Truth truth = NormalTruth.MonotoneDecreasingMeansExample();
		Run(sampler, stopSeq, truth);
	}
	public static void testA9() {
		SamplingRule sampler = new InfoCollection.IndependentNormalVarianceKnown.KnowledgeGradient();
		StoppingRuleSeq stopSeq = InfoCollection.IndependentNormalVarianceKnown.EOCBonfStoppingRule.Seq();
		Truth truth = NormalTruth.MonotoneDecreasingMeansExample();
		Run(sampler, stopSeq, truth);
	}
	public static void testA10() {
		SamplingRule sampler = new InfoCollection.IndependentNormalVarianceKnown.ChIn();
		StoppingRuleSeq stopSeq = InfoCollection.IndependentNormalVarianceKnown.KGStoppingRule.Seq();
		Truth truth = NormalTruth.MonotoneDecreasingMeansExample();
		Run(sampler, stopSeq, truth);
	}
	public static void testA11() {
		SamplingRule sampler = new InfoCollection.IndependentNormalVarianceKnown.ChIn();
		StoppingRuleSeq stopSeq = FixedStop.Seq();
		Truth truth = NormalTruth.MonotoneDecreasingMeansExample();
		Run(sampler, stopSeq, truth);
	}
	public static void testA12() {
		SamplingRule sampler = new InfoCollection.IndependentNormalVarianceKnown.ChIn();
		StoppingRuleSeq stopSeq = InfoCollection.IndependentNormalVarianceKnown.EOCBonfStoppingRule.Seq();
		Truth truth = NormalTruth.MonotoneDecreasingMeansExample();
		Run(sampler, stopSeq, truth);
	}



	/* 
	 * These are the same as tests 1-12, except that we simulate the
	 * variance-known policy by using the variance-unknown policy with a
	 * prior belief concentrated on the true precision.  When we compare
	 * these to tests A1-A12, the results should be very similar.
	 */
	public static void testB1() {
		NormalTruth truth = NormalTruth.SlippageExample();
		Belief b0 = new Belief(truth.SamplingPrecision());
		//System.out.println("b0=" + b0.toString());
		SamplingRule sampler = new LL1(b0);
		StoppingRuleSeq stopSeq = KGStoppingRule.Seq(b0);
		Run(sampler, stopSeq, truth);
	}
	public static void testB2() {
		NormalTruth truth = NormalTruth.SlippageExample();
		Belief b0 = new Belief(truth.SamplingPrecision());
		SamplingRule sampler = new LL1(b0);
		StoppingRuleSeq stopSeq = FixedStop.Seq();
		Run(sampler, stopSeq, truth);
	}
	public static void testB3() {
		NormalTruth truth = NormalTruth.SlippageExample();
		Belief b0 = new Belief(truth.SamplingPrecision());
		SamplingRule sampler = new LL1(b0);
		StoppingRuleSeq stopSeq = EOCBonfStoppingRule.Seq(b0);
		Run(sampler, stopSeq, truth);
	}
	public static void testB4() {
		NormalTruth truth = NormalTruth.SlippageExample();
		Belief b0 = new Belief(truth.SamplingPrecision());
		SamplingRule sampler = new ChIn(b0);
		StoppingRuleSeq stopSeq = KGStoppingRule.Seq(b0);
		Run(sampler, stopSeq, truth);
	}
	public static void testB5() {
		NormalTruth truth = NormalTruth.SlippageExample();
		Belief b0 = new Belief(truth.SamplingPrecision());
		SamplingRule sampler = new ChIn(b0);
		StoppingRuleSeq stopSeq = FixedStop.Seq();
		Run(sampler, stopSeq, truth);
	}
	public static void testB6() {
		NormalTruth truth = NormalTruth.SlippageExample();
		Belief b0 = new Belief(truth.SamplingPrecision());
		SamplingRule sampler = new ChIn(b0);
		StoppingRuleSeq stopSeq = EOCBonfStoppingRule.Seq(b0);
		Run(sampler, stopSeq, truth);
	}
	public static void testB7() {
		NormalTruth truth = NormalTruth.SlippageExample();
		Belief b0 = new Belief(truth.SamplingPrecision());
		SamplingRule sampler = new LL1(b0);
		StoppingRuleSeq stopSeq = KGStoppingRule.Seq(b0);
		Run(sampler, stopSeq, truth);
	}
	public static void testB8() {
		NormalTruth truth = NormalTruth.SlippageExample();
		Belief b0 = new Belief(truth.SamplingPrecision());
		SamplingRule sampler = new LL1(b0);
		StoppingRuleSeq stopSeq = FixedStop.Seq();
		Run(sampler, stopSeq, truth);
	}
	public static void testB9() {
		NormalTruth truth = NormalTruth.SlippageExample();
		Belief b0 = new Belief(truth.SamplingPrecision());
		SamplingRule sampler = new LL1(b0);
		StoppingRuleSeq stopSeq = EOCBonfStoppingRule.Seq(b0);
		Run(sampler, stopSeq, truth);
	}
	public static void testB10() {
		NormalTruth truth = NormalTruth.SlippageExample();
		Belief b0 = new Belief(truth.SamplingPrecision());
		SamplingRule sampler = new ChIn(b0);
		StoppingRuleSeq stopSeq = KGStoppingRule.Seq(b0);
		Run(sampler, stopSeq, truth);
	}
	public static void testB11() {
		NormalTruth truth = NormalTruth.SlippageExample();
		Belief b0 = new Belief(truth.SamplingPrecision());
		SamplingRule sampler = new ChIn(b0);
		StoppingRuleSeq stopSeq = FixedStop.Seq();
		Run(sampler, stopSeq, truth);
	}
	public static void testB12() {
		NormalTruth truth = NormalTruth.SlippageExample();
		Belief b0 = new Belief(truth.SamplingPrecision());
		SamplingRule sampler = new ChIn(b0);
		StoppingRuleSeq stopSeq = EOCBonfStoppingRule.Seq(b0);
		Run(sampler, stopSeq, truth);
	}


	// This is a nice typical run.
	static int seed = 14;
	public static void testC1() {
		SamplingRule sampler = new LL1();
		StoppingRuleSeq stopSeq = KGStoppingRule.Seq();
		Truth truth = NormalTruth.RandomConfiguration(seed);
		System.out.printf("# RPI(seed=%d)\n", seed);
		Run(sampler, stopSeq, truth);
	}
	public static void testC2() {
		SamplingRule sampler = new LL1();
		StoppingRuleSeq stopSeq = FixedStop.Seq();
		Truth truth = NormalTruth.RandomConfiguration(seed);
		System.out.printf("# RPI(seed=%d)\n", seed);
		Run(sampler, stopSeq, truth);
	}
	public static void testC3() {
		SamplingRule sampler = new LL1();
		StoppingRuleSeq stopSeq = EOCBonfStoppingRule.Seq();
		Truth truth = NormalTruth.RandomConfiguration(seed);
		System.out.printf("# RPI(seed=%d)\n", seed);
		Run(sampler, stopSeq, truth);
	}
	public static void testC4() {
		SamplingRule sampler = new ChIn();
		StoppingRuleSeq stopSeq = KGStoppingRule.Seq();
		Truth truth = NormalTruth.RandomConfiguration(seed);
		System.out.printf("# RPI(seed=%d)\n", seed);
		Run(sampler, stopSeq, truth);
	}
	public static void testC5() {
		SamplingRule sampler = new ChIn();
		StoppingRuleSeq stopSeq = FixedStop.Seq();
		Truth truth = NormalTruth.RandomConfiguration(seed);
		System.out.printf("# RPI(seed=%d)\n", seed);
		Run(sampler, stopSeq, truth);
	}
	public static void testC6() {
		SamplingRule sampler = new ChIn();
		StoppingRuleSeq stopSeq = EOCBonfStoppingRule.Seq();
		Truth truth = NormalTruth.RandomConfiguration(seed);
		System.out.printf("# RPI(seed=%d)\n", seed);
		Run(sampler, stopSeq, truth);
	}


	/* Run LL with a fixed value of the block-size B (also called tau). */
	public static void testD2() { testD(2); }
	public static void testD3() { testD(3); }
	public static void testD4() { testD(4); }
	public static void testD5() { testD(5); }
	public static void testD6() { testD(6); }
	public static void testD7() { testD(7); }
	public static void testD8() { testD(8); }
	public static void testD9() { testD(9); }
	public static void testD10() { testD(10); }
	public static void testD(int B) {
		SamplingRule sampler = new ChIn(6, B);
		StoppingRuleSeq stopSeq = EOCBonfStoppingRule.Seq();
		Truth truth = NormalTruth.SlippageExample();
		Run(sampler, stopSeq, truth);
	}

}
