// not part of package InfoCollection
import InfoCollection.SamplingRule;
import InfoCollection.StoppingRuleSeq;
import InfoCollection.Simulator;
import InfoCollection.TestExamples;
import InfoCollection.NormalTruth;
import InfoCollection.Truth;
import InfoCollection.FixedStop;
import InfoCollection.ThresholdStoppingRule;
import InfoCollection.IndependentNormalVarianceUnknown.Belief;
import InfoCollection.IndependentNormalVarianceUnknown.LL1;
import InfoCollection.IndependentNormalVarianceUnknown.ChIn;
import InfoCollection.IndependentNormalVarianceUnknown.KGStoppingRule;
import InfoCollection.IndependentNormalVarianceUnknown.EOCBonfStoppingRule;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import java.util.Random;

public class Sim20080303 {

	/* Note that it doesn't matter what actual threshold one uses for stop,
	 * because we just run for a fixed number N iterations. */
	private static void Run(SamplingRule policy, Truth truth, ThresholdStoppingRule stop) {
		//int nruns = 100000; // for production
		int nruns = 1000; // for testing
		int N = 400;
		double result[][];

		System.out.println("# Policy=" + policy);
		System.out.println("# Truth=" + truth);
		System.out.println("# StoppingRule=" + stop);
		System.out.println("# n\tStopValue");

		Random rnd = new Random();
		try {
			policy.Start(truth.M());
			stop.Start(truth.M());
			for (int n=0; n<N; n++) {
				int x = policy.GetMeasurementDecision();
				double y = truth.Sample(x,rnd);
				policy.RecordMeasurement(x,y);
				stop.RecordMeasurement(x,y);
				System.out.printf("%d %f\n", n, stop.GetCurrentValue());
			}

		} catch (Exception e) {
			System.err.println("Internal Error");
			e.printStackTrace();
			return;
		}
	}

	public static void main(String args[]) {
		if (args.length == 1) {
			runtest(args[0]);
		} else
			System.err.println("usage: Sim200800303 test_num");
	}

	/*
	 * Run test number n. This is a function named "testn", where n
	 * is filled in with the test number.  Returns 0 if the test succeeded,
	 * -1 if the test was not found, and -2 if the test was found but
	 * failed.  Also prints an error message if it failed.  It may also
	 * fail for some unexpected reason, in which case it throws that
	 * exception.
	 */
	public static void runtest(String suffix) {
		Method method = null;
		String testname = "test" + suffix;
		try {
			method = Sim20080303.class.getDeclaredMethod(testname, new Class[] {});
		} catch(NoSuchMethodException e) {
			System.err.printf("%s not found\n", testname);
		}
		try {
			method.invoke(null);
		} catch(InvocationTargetException target_e) {
			target_e.getCause().printStackTrace();
			System.err.println(testname + " failed");
		} catch(IllegalAccessException e) {
			e.printStackTrace();
			System.err.println(testname + " access error");
		}
	}


	/*
	 * Let's test the EDM configuration for EOCBonf and KG stopping values. 
	 */
	public static void test1() {
		SamplingRule sampler = new LL1();
		ThresholdStoppingRule stop = new KGStoppingRule(1);
		Truth truth = NormalTruth.ExponentialDecreasingMeansExample();
		Run(sampler, truth, stop);
	}
	public static void test2() {
		SamplingRule sampler = new LL1();
		ThresholdStoppingRule stop = new EOCBonfStoppingRule(1);
		Truth truth = NormalTruth.ExponentialDecreasingMeansExample();
		Run(sampler, truth, stop);
	}

	/* Here is the slippage configuration. */
	public static void test3() {
		SamplingRule sampler = new LL1();
		ThresholdStoppingRule stop = new KGStoppingRule(1);
		Truth truth = NormalTruth.SlippageExample();
		Run(sampler, truth, stop);
	}

}
