// not part of package InfoCollection
import InfoCollection.SamplingRule;
import InfoCollection.StoppingRuleSeq;
import InfoCollection.Simulator;
import InfoCollection.TestExamples;
import InfoCollection.NormalTruth;
import InfoCollection.Truth;
import InfoCollection.FixedStop;
import InfoCollection.IndependentNormalVarianceUnknown.Belief;
import InfoCollection.IndependentNormalVarianceUnknown.LL1;
import InfoCollection.IndependentNormalVarianceUnknown.ChIn;
import InfoCollection.IndependentNormalVarianceUnknown.KGStoppingRule;
import InfoCollection.IndependentNormalVarianceUnknown.EOCBonfStoppingRule;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;

public class RPI {
	/* This is the basic code for running the experiment. */
	private static void Run(SamplingRule policy, StoppingRuleSeq stopSeq, Truth truth) {
		int nruns = 100000; // for production
		//int nruns = 10000; // for testing
		int maxN = 2000;
		double result[][];

		/* The description of truth will have newlines in it, so we
		 * have to replace these with comments. */
		String truthDescrip = "# " + truth.toString();
		truthDescrip = truthDescrip.replaceAll("\n", "\n# ");

		System.out.printf("# nruns=%d maxN=%d\n", nruns, maxN);
		System.out.println("# Policy=" + policy);
		System.out.println(truthDescrip);
		System.out.println("# Stopping Rule=" + stopSeq);
		System.out.println("# Stopping Rule=" + stopSeq);
		System.out.println("# E[N]\tlog10(E[OC])\ttimeouts");

		try {
			result = Simulator.LossWhenStopped(policy, truth, stopSeq, nruns, maxN);
		} catch (Exception e) {
			System.err.println("Internal Error");
			e.printStackTrace();
			return;
		}
			
		for (int s=0; s<stopSeq.Length(); s++)
			System.out.printf("%.6f\t%.6f\t%d\n",
					result[s][1], StrictMath.log10(result[s][0]), (int)result[s][2]);
	}

	public static void main(String args[]) {
		if (args.length == 2) {
			int seed = Integer.parseInt(args[1]);
			runtest(args[0],seed);
		} else
			System.err.println("usage: RPI test_num seed");
	}

	/*
	 * Run test number n. This is a function named "testn", where n
	 * is filled in with the test number.  Returns 0 if the test succeeded,
	 * -1 if the test was not found, and -2 if the test was found but
	 * failed.  Also prints an error message if it failed.  It may also
	 * fail for some unexpected reason, in which case it throws that
	 * exception.
	 */
	public static void runtest(String suffix, int seed) {
		Method method = null;
		String testname = "test" + suffix;
		Class [] argdef = new Class[1];
		argdef[0] = Integer.TYPE;
		try {
			method = RPI.class.getDeclaredMethod(testname, argdef);
		} catch(NoSuchMethodException e) {
			System.err.printf("%s not found\n", testname);
			return;
		}
		//Object args[] = { seed }; 
		try {
			method.invoke(null, seed);
		} catch(InvocationTargetException target_e) {
			target_e.getCause().printStackTrace();
			System.err.println(testname + " failed");
		} catch(IllegalAccessException e) {
			e.printStackTrace();
			System.err.println(testname + " access error");
		}
	}


	public static void test1(int seed) {
		SamplingRule sampler = new LL1();
		StoppingRuleSeq stopSeq = KGStoppingRule.Seq();
		Truth truth = NormalTruth.RandomConfiguration(seed);
		System.out.printf("# RPI(seed=%d)\n", seed);
		Run(sampler, stopSeq, truth);
	}
	public static void test2(int seed) {
		SamplingRule sampler = new LL1();
		StoppingRuleSeq stopSeq = FixedStop.Seq();
		Truth truth = NormalTruth.RandomConfiguration(seed);
		System.out.printf("# RPI(seed=%d)\n", seed);
		Run(sampler, stopSeq, truth);
	}
	public static void test3(int seed) {
		SamplingRule sampler = new LL1();
		StoppingRuleSeq stopSeq = EOCBonfStoppingRule.Seq();
		Truth truth = NormalTruth.RandomConfiguration(seed);
		System.out.printf("# RPI(seed=%d)\n", seed);
		Run(sampler, stopSeq, truth);
	}
	public static void test4(int seed) {
		SamplingRule sampler = new ChIn();
		StoppingRuleSeq stopSeq = KGStoppingRule.Seq();
		Truth truth = NormalTruth.RandomConfiguration(seed);
		System.out.printf("# RPI(seed=%d)\n", seed);
		Run(sampler, stopSeq, truth);
	}
	public static void test5(int seed) {
		SamplingRule sampler = new ChIn();
		StoppingRuleSeq stopSeq = FixedStop.Seq();
		Truth truth = NormalTruth.RandomConfiguration(seed);
		System.out.printf("# RPI(seed=%d)\n", seed);
		Run(sampler, stopSeq, truth);
	}
	public static void test6(int seed) {
		SamplingRule sampler = new ChIn();
		StoppingRuleSeq stopSeq = EOCBonfStoppingRule.Seq();
		Truth truth = NormalTruth.RandomConfiguration(seed);
		System.out.printf("# RPI(seed=%d)\n", seed);
		Run(sampler, stopSeq, truth);
	}

}
